package betterquesting.api2.storage;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.BitSet;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;

/* loaded from: betterquesting_api2_storage.jar:betterquesting/api2/storage/SimpleDatabase.class */
public abstract class SimpleDatabase<T> implements IDatabase<T> {
    private final TreeMap<Integer, T> mapDB = new TreeMap<>();
    private final BitSet idMap = new BitSet();
    private List<DBEntry<T>> refCache = null;

    @Override // betterquesting.api2.storage.IDatabase
    public synchronized int nextID() {
        return this.idMap.nextClearBit(0);
    }

    @Override // betterquesting.api2.storage.IDatabase
    public synchronized DBEntry<T> add(int id, T value) {
        if (value == null) {
            throw new NullPointerException("Value cannot be null");
        }
        if (id < 0) {
            throw new IllegalArgumentException("ID cannot be negative");
        }
        if (this.mapDB.putIfAbsent(Integer.valueOf(id), value) == null) {
            this.idMap.set(id);
            this.refCache = null;
            return new DBEntry<>(id, value);
        }
        throw new IllegalArgumentException("ID or value is already contained within database");
    }

    @Override // betterquesting.api2.storage.IDatabase
    public synchronized boolean removeID(int key) {
        if (key >= 0 && this.mapDB.remove(Integer.valueOf(key)) != null) {
            this.idMap.clear(key);
            this.refCache = null;
            return true;
        }
        return false;
    }

    @Override // betterquesting.api2.storage.IDatabase
    public synchronized boolean removeValue(T value) {
        return value != null && removeID(getID(value));
    }

    @Override // betterquesting.api2.storage.IDatabase
    public synchronized int getID(T value) {
        if (value == null) {
            return -1;
        }
        for (DBEntry<T> entry : getEntries()) {
            if (entry.getValue() == value) {
                return entry.getID();
            }
        }
        return -1;
    }

    @Override // betterquesting.api2.storage.IDatabase
    public synchronized T getValue(int id) {
        if (id < 0 || this.mapDB.size() <= 0) {
            return null;
        }
        return this.mapDB.get(Integer.valueOf(id));
    }

    @Override // betterquesting.api2.storage.IDatabase
    public synchronized int size() {
        return this.mapDB.size();
    }

    @Override // betterquesting.api2.storage.IDatabase
    public synchronized void reset() {
        this.mapDB.clear();
        this.idMap.clear();
        this.refCache = Collections.emptyList();
    }

    @Override // betterquesting.api2.storage.IDatabase
    public synchronized List<DBEntry<T>> getEntries() {
        if (this.refCache == null) {
            List<DBEntry<T>> temp = new ArrayList<>();
            for (Map.Entry<Integer, T> entry : this.mapDB.entrySet()) {
                temp.add(new DBEntry<>(entry.getKey().intValue(), entry.getValue()));
            }
            this.refCache = Collections.unmodifiableList(temp);
        }
        return this.refCache;
    }

    @Override // betterquesting.api2.storage.IDatabase
    public synchronized List<DBEntry<T>> bulkLookup(int... keys) {
        if (keys.length <= 0) {
            return Collections.emptyList();
        }
        int[] sortedKeys = new int[keys.length];
        System.arraycopy(keys, 0, sortedKeys, 0, keys.length);
        Arrays.sort(sortedKeys);
        List<DBEntry<T>> subList = new ArrayList<>();
        int n = 0;
        for (DBEntry<T> entry : getEntries()) {
            while (n < sortedKeys.length && sortedKeys[n] < entry.getID()) {
                n++;
            }
            if (n >= sortedKeys.length) {
                break;
            }
            if (sortedKeys[n] == entry.getID()) {
                subList.add(entry);
            }
        }
        return subList;
    }
}
